@@ -219,7 +219,8 @@ class DistGitRepo(object):
 
     def source_path(self):
         """
-        :return: Returns the directory containing the source which should be used to populate distgit.
+        :return: Returns the directory containing the source which should be used to populate distgit. This includes
+                the source.path subdirectory if the metadata includes one.
         """
 
         source_root = self.runtime.resolve_source(self.metadata)
@@ -232,6 +233,14 @@ class DistGitRepo(object):
         assertion.isdir(path, "Unable to find path for source [%s] for config: %s" % (path, self.metadata.config_filename))
         return path
 
+    def source_repo_path(self):
+        """
+        :return: Returns the directory containing the root of the cloned source repo.
+        """
+        path = self.runtime.resolve_source(self.metadata)
+        assertion.isdir(path, "Unable to find path for source [%s] for config: %s" % (path, self.metadata.config_filename))
+        return path
+
     def commit(self, commit_message, log_diff=False):
         if self.runtime.local:
             return ''  # no commits if local
@@ -454,7 +463,7 @@ class ImageDistGitRepo(DistGitRepo):
                 self.org_release = dfp.labels.get("release")  # occasionally no release given
 
     def push_image(self, tag_list, push_to_defaults, additional_registries=[], version_release_tuple=None,
-                   push_late=False, dry_run=False):
+                   push_late=False, dry_run=False, registry_config_dir=None, filter_by_os=None):
         """
         Pushes the most recent image built for this distgit repo. This is
         accomplished by looking at the 'version' field in the Dockerfile or
@@ -496,6 +505,12 @@ class ImageDistGitRepo(DistGitRepo):
         if not push_names:
             return (self.metadata.distgit_key, True)
 
+        # get registry_config_json file must before with Dir(self.distgit_dir)
+        # so that relative path or env like DOCKER_CONFIG will not pointed to distgit dir.
+        registry_config_file = ''
+        if registry_config_dir is not None:
+            registry_config_file = util.get_docker_config_json(registry_config_dir)
+
         with Dir(self.distgit_dir):
 
             if version_release_tuple:
@@ -584,8 +599,12 @@ class ImageDistGitRepo(DistGitRepo):
 
                     with io.open(push_config, 'w', encoding="utf-8") as pc:
                         pc.write('\n'.join(all_push_urls))
-
-                    mirror_cmd = 'oc image mirror --filter-by-os=amd64 {} {} --filename={}'.format(dr, insecure, push_config)
+                    mirror_cmd = 'oc image mirror '
+                    if filter_by_os is not None:
+                        mirror_cmd += "--filter-by-os={}".format(filter_by_os)
+                    mirror_cmd += " {} {} --filename={}".format(dr, insecure, push_config)
+                    if registry_config_file != '':
+                        mirror_cmd += f" --registry-config={registry_config_file}"
 
                     if dry_run:  # skip everything else if dry run
                         continue
@@ -656,7 +675,7 @@ class ImageDistGitRepo(DistGitRepo):
 
     def build_container(
             self, profile, push_to_defaults, additional_registries, terminate_event,
-            scratch=False, retries=3, realtime=False, dry_run=False):
+            scratch=False, retries=3, realtime=False, dry_run=False, registry_config_dir=None, filter_by_os=None):
         """
         This method is designed to be thread-safe. Multiple builds should take place in brew
         at the same time. After a build, images are pushed serially to all mirrors.
@@ -785,7 +804,7 @@ class ImageDistGitRepo(DistGitRepo):
             with self.runtime.mutex:
                 self.push_status = False
                 try:
-                    self.push_image([], push_to_defaults, additional_registries, version_release_tuple=(push_version, push_release))
+                    self.push_image([], push_to_defaults, additional_registries, version_release_tuple=(push_version, push_release), registry_config_dir=registry_config_dir, filter_by_os=filter_by_os)
                     self.push_status = True
                 except Exception as push_e:
                     self.logger.info("Error during push after successful build: %s" % str(push_e))
@@ -1786,29 +1805,7 @@ class ImageDistGitRepo(DistGitRepo):
             if self.private_fix is None and self.metadata.public_upstream_branch:
                 self.private_fix = not util.is_commit_in_public_upstream(self.source_full_sha, self.metadata.public_upstream_branch, source_dir)
 
-            # If this is a go project, parse the Godeps for points of interest
-            godeps_file = pathlib.Path(self.source_path(), 'Godeps', 'Godeps.json')
-            if godeps_file.is_file():
-                try:
-                    with godeps_file.open('r', encoding='utf-8') as f:
-                        godeps = json.load(f)
-                        # Reproduce https://github.com/openshift/origin/blob/6f457bc317f8ca8e514270714db6597ec1cb516c/hack/lib/build/version.sh#L82
-                        # Example of what we are after: https://github.com/openshift/origin/blob/6f457bc317f8ca8e514270714db6597ec1cb516c/Godeps/Godeps.json#L10-L15
-                        for dep in godeps.get('Deps', []):
-                            if dep.get('ImportPath', '') == 'k8s.io/kubernetes/pkg/api':
-                                raw_kube_version = dep.get('Comment', '')  # e.g. v1.14.6-152-g117ba1f
-                                # drop release information.
-                                base_kube_version = raw_kube_version.split('-')[0]
-                                kube_version_fields = base_kube_version.lstrip('v').split('.')  # v1.17.1-152-g117ba1f => [ '1', '17', '1' ]
-                                # For historical consistency with tito's flow, we add +OS_GIT_COMMIT[:7] to the kube version
-                                self.env_vars_from_source['KUBE_GIT_VERSION'] = f"v{'.'.join(kube_version_fields)}+{self.source_full_sha[:7]}"
-                                self.env_vars_from_source['KUBE_GIT_COMMIT'] = dep.get('Rev', '')
-                                self.env_vars_from_source['KUBE_GIT_MAJOR'] = '0' if len(kube_version_fields) < 1 else kube_version_fields[0]
-                                godep_kube_minor = '0' if len(kube_version_fields) < 2 else kube_version_fields[1]
-                                self.env_vars_from_source['KUBE_GIT_MINOR'] = f'{godep_kube_minor}+'  # For historical reasons, add a + since OCP patches its vendor kube.
-                except:
-                    self.runtime.logger.error(f'Error parsing godeps {str(godeps_file)}')
-                    traceback.print_exc()
+            self.env_vars_from_source.update(self.metadata.extract_kube_env_vars())
 
         # See if the config is telling us a file other than "Dockerfile" defines the
         # distgit image content.
