@@ -322,12 +322,22 @@ class Repos(object):
         return set(result)
 
     def validate_content_sets(self):
+        # Determine repos that have no content sets defined at all; we will give these a pass if nothing tries to use them.
+        # This is one reason to accept it if no content_set is defined at all: https://github.com/openshift/ocp-build-data/pull/594
+        content_set_defined = {}
+        for name, repo in self._repos.items():
+            content_set_defined[name] = False
+            for arch in self._arches + ['default']:
+                if repo._data.content_set[arch] is not Missing:
+                    content_set_defined[name] = True
+
         invalid = []
         for arch in self._arches:
             cs_names = {}
             for name, repo in self._repos.items():
-                cs = repo.content_set(arch)
-                cs_names[name] = cs
+                if content_set_defined[name]:
+                    cs = repo.content_set(arch)
+                    cs_names[name] = cs
 
             arch_cs_values = list(cs_names.values())
             if arch_cs_values:
